# -*- coding: utf-8 -*-

from unittest import TestCase


class TestPushApp(TestCase):
    """
    Test push app with call plugin instance
    """

    def runTest(self):
        from noseapp import NoseApp
        from noseapp import AppPlugin

        plugin = AppPlugin()
        self.assertIsNone(plugin.app)

        plugin.init_app(NoseApp('test'))
        self.assertIsInstance(plugin, AppPlugin)
        self.assertIsInstance(plugin.app, NoseApp)



# -*- coding: utf-8 -*-

import logging
import unittest
from types import FunctionType

from noseapp.core import loader
from noseapp.core import extensions
from noseapp.case.base import TestCase
from noseapp.suite.context import SuiteContext
from noseapp.case.base import make_test_case_class_from_function


logger = logging.getLogger(__name__)


def suite_is_mount(suite):
    """
    If suite is mounting to app then True else False

    :param suite: suite instance
    :type suite: Suite

    :rtype: bool
    """
    return bool(suite.of_app)


class Suite(object):
    """
    Base Suite class for group or one TestCase.

    Simple example:

        suite = Suite(__name__)

        suite.register
        def my_test_case(case):
            case.assertTrue(True)
    """

    # Constant is pre set for require param
    DEFAULT_REQUIRE = None

    # This class is base class for make test case from function
    test_case_class = TestCase

    def __init__(self, name, require=None, context=None):
        """
        :param name: suite name
        :type name: str
        :param require: extension names list
        :type require: list
        :param context: context instance
        :type context: noseapp.suite.context.SuiteContext
        """
        # Suite name. Must be str only
        self.__name = name
        # of app name
        self.__of_app = None
        # Was suite built? True or False
        self.__is_build = False

        # Context of this instance
        self.__context = context or SuiteContext(
            list(self.DEFAULT_REQUIRE or []) + list(require or []),
        )

        # Push callbacks to context. They'll be called with nose suite.
        self.add_before(lambda: self.setUp())
        self.add_after(lambda: self.tearDown())

    @property
    def name(self):
        if self.__of_app:
            return '{}.{}'.format(
                self.__of_app, self.__name,
            )

        return self.__name

    @property
    def of_app(self):
        return self.__of_app

    @property
    def context(self):
        return self.__context

    @property
    def is_build(self):
        return self.__is_build

    @property
    def test_cases(self):
        return self.__context.test_cases

    @property
    def require(self):
        return self.__context.require

    def mount_to_app(self, app):
        """
        :type app: noseapp.app.base.NoseApp
        """
        if suite_is_mount(self):
            raise RuntimeError(
                '{} already mount to <NoseApp {}>. '.format(self, self.__of_app),
            )

        self.__of_app = app.name

        app.context.add_suite(self)

    @property
    def TestCase(self):
        """
        :rtype: noseapp.case.base.TestCase
        """
        return self.test_case_class

    def setUp(self):
        """
        Callback. Will be called run before run suite.
        """
        pass

    def tearDown(self):
        """
        Callback. Will be called run after run suite.
        """
        pass

    @property
    def skip(self):
        """
        :rtype: unittest.skip
        """
        return unittest.skip

    @property
    def skip_if(self):
        """
        :rtype: unittest.skipIf
        """
        return unittest.skipIf

    @property
    def skip_unless(self):
        """
        :rtype: unittest.skipUnless
        """
        return unittest.skipUnless

    # unit test style
    skipIf = skip_if
    skipUnless = skip_unless

    def add_pre_run(self, func):
        """
        Set function for pre run test case.

        example:
            suite.add_pre_run(lambda case: case.do_something())
        """
        self.__context.add_pre_run_handler(func)
        return func

    def add_post_run(self, func):
        """
        Set function for post run test case.

        example:
            suite.add_post_run(lambda case: case.do_something())
        """
        self.__context.add_post_run_handler(func)
        return func

    def add_before(self, func):
        """
        Set setup callback for suite prepare
        """
        self.__context.add_setup(func)
        return func

    def add_after(self, func):
        """
        Set teardown callback for suite prepare
        """
        self.__context.add_teardown(func)
        return func

    def ext(self, name):
        """
        Get extension by name.
        Extensions will be available after built suite.

        :param name: extension name
        """
        if not self.__is_build:
            raise RuntimeError(
                'Extension is not allow. Suite is not building.',
            )

        if name not in self.__context.require:
            raise LookupError(
                'Extension "{}" is not required'.format(name),
            )

        return self.__context.extensions.get(name)

    def register(self, cls=None, **kwargs):
        """
        Register test case on context

        :param cls: test case class
        :type cls: noseapp.case.TestCase or function

        :param simple: use for simple test function
        :type simple: bool

        :param skip: if to set then skip case
        :type skip: str

        :rtype: cls
        """
        if not cls and not kwargs:
            raise TypeError('cls param or **kwargs is required')
        elif cls and kwargs:
            raise TypeError('**kwargs can not be used with cls param')

        def wrapped(_class, simple=False, skip=None):
            if type(_class) == FunctionType:
                _class = make_test_case_class_from_function(
                    _class,
                    simple=simple,
                    base_class=self.test_case_class,
                )

            if skip:
                unittest.skip(skip)(_class)

            self.__context.add_test_case(_class)

            logger.debug(
                'Register test case "{}" in {}'.format(
                    _class.__name__, repr(self),
                ),
            )

            return _class

        if cls:
            return wrapped(cls)

        def wrapper(_class):
            return wrapped(_class, **kwargs)

        return wrapper

    def get_map(self):
        """
        Get map of test classes

        {
            'class name': {
                'cls': 'link to class object',
                'tests': {
                    'method name': 'link to class method',
                },
            },
        }

        :return: dict
        """
        mp = {}

        for case in self.__context.test_cases:
            mp[case.__name__] = {
                'cls': case,
                'tests': dict(
                    (atr, getattr(case, atr))
                    for atr in dir(case)
                    if atr.startswith(loader.TEST_NAME_PREFIX)
                    or
                    atr == loader.DEFAULT_TEST_NAME,
                ),
            }

        return mp

    def __call__(self,
                 program_data,
                 shuffle=None,
                 case_name=None,
                 method_name=None):
        """
        Build suite

        :type program_data: noseapp.core.program.ProgramData

        :param shuffle: callable object for randomize test case list
        :param case_name: test name for build
        :param method_name: test case method name for build
        """
        if not self.__of_app:
            raise RuntimeError('Suite can not be building, not mount to app')

        self.__is_build = True

        for ext_name in self.context.require:
            ext = extensions.get(ext_name)
            self.context.add_extension(ext_name, ext)

        if callable(shuffle):
            shuffle(self.__context.test_cases)

        def make_suites():
            suites = []

            if case_name:
                case = loader.load_case_from_suite(case_name, self)
                tests = loader.load_tests_from_test_case(
                    case.mount_to_suite(self),
                    method_name=method_name,
                )

                suites.append(
                    program_data.suite_class(
                        tests,
                        context=case,
                        config=program_data.config,
                        resultProxy=program_data.result_proxy_factory,
                        pre_run_handlers=self.__context.pre_run_handlers,
                        post_run_handlers=self.__context.post_run_handlers,
                    ),
                )
            else:
                for case in self.__context.test_cases:
                    tests = loader.load_tests_from_test_case(
                        case.mount_to_suite(self),
                    )

                    suites.append(
                        program_data.suite_class(
                            tests,
                            context=case,
                            config=program_data.config,
                            resultProxy=program_data.result_proxy_factory,
                            pre_run_handlers=self.__context.pre_run_handlers,
                            post_run_handlers=self.__context.post_run_handlers,
                        ),
                    )

            return suites

        return program_data.suite_class(
            make_suites(),
            context=self.__context,
            config=program_data.config,
        )

    def __repr__(self):
        return '<Suite {}>'.format(self.name)



# -*- coding: utf-8 -*-

"""
Application is entry point to your test program.
You can configure and expand your program from application.
"""

import os
import sys
import logging
from contextlib import contextmanager

from noseapp.core import loader
from noseapp.core import extensions
from noseapp.core import TestProgram
from noseapp.app.context import AppContext
from noseapp.app.context import app_callback
from noseapp.app.context import merge_context
from noseapp.core.factory import ClassFactory
from noseapp.core.collector import CollectSuite
from noseapp.app.config import Config as AppConfig
from noseapp.core.constants import CONFIG_CHECKOUT_ENV
from noseapp.plugins.configure import AppConfigurePlugin


logger = logging.getLogger(__name__)


DEFAULT_PLUGINS = [
    AppConfigurePlugin(),
]

APP_CONFIG = os.getenv(CONFIG_CHECKOUT_ENV, None)


class NoseApp(object):
    """
    Base application class

    Simple example:
        app = NoseApp('my_app')
        app.load_suites('/path/to/suites')
        app.run()

    Application can be master and sub application:

        sub_app = NoseApp.as_sub_app('sub')
        master_app = NoseApp.as_master_app('master', sub_app)

        sub_app.load_suites('/path/to/suites')
        master_app.load_suites('/path/to/suites', merge_suites=True)

        master_app.run()
    """

    config_class = AppConfig
    program_class = TestProgram
    class_factory = ClassFactory
    collector_class = CollectSuite

    def __init__(self,
                 name,
                 argv=None,
                 exit=True,
                 config=None,
                 plugins=None,
                 context=None,
                 sub_apps=None,
                 suites_path=None,
                 is_sub_app=False):
        """
        :type name: str
        :type argv: list
        :type exit: bool
        :type config: str
        :type plugins: list or tuple
        :type context: noseapp.app.context.AppContext
        :type is_sub_app: bool
        :type sub_apps: list or tuple
        """
        if sub_apps and is_sub_app:
            raise RuntimeError(
                '"{}" can not be maser app. The application is sub app.'.format(self),
            )

        # Initialization config storage
        self.config = self.config_class.from_path(APP_CONFIG or config)

        # Application name. Must be str only.
        self.__name = name

        # Command line options. Will be set after test program initialization.
        self.__options = None

        # If app is sub app that's True else False
        self.__is_sub_app = is_sub_app

        # Sub application list
        self.__sub_apps = list(sub_apps or [])

        # Context will be inject to nose suite
        self.__context = context or AppContext()

        # Add callbacks to context
        self.__context.add_setup(lambda: self.setUp())
        self.__context.add_teardown(lambda: self.tearDown())

        # Saved support nose plugins and add supported self plugins.
        self.__context.plugins.extend(list(plugins or []))

        if self.is_master_app:
            # Default plugins for master app only
            self.__context.plugins.extend(DEFAULT_PLUGINS)
            # Merge context of sub applications
            merge_context(self,
                          merge_setup=True,
                          merge_suites=True,
                          merge_plugins=True,
                          merge_teardown=True)
            # Program object initialization. Will be run later.
            self.__test_program = self.program_class(self, argv=argv, exit=exit)

            # If we are having path to suites dir then make load suites
            if suites_path is not None:
                self.load_suites(suites_path)

    @classmethod
    def as_master_app(cls, name, *sub_apps, **kwargs):
        """
        Create application as master application
        """
        kwargs.update(is_sub_app=False)
        kwargs.setdefault('sub_apps', sub_apps)

        return cls(name, **kwargs)

    @classmethod
    def as_sub_app(cls, name, **kwargs):
        """
        Create application as sub application
        """
        kwargs.update(
            sub_apps=None,
            is_sub_app=True,
        )

        return cls(name, **kwargs)

    def setUpApp(self):
        """
        Callback for initialize application.
        Will be call after initialize instance.
        """
        pass

    def tearDownApp(self):
        """
        Callback for destruct application.
        Will be called after exit from test program.
        """
        pass

    def setUp(self):
        """
        Callback will be called before run application
        """
        pass

    def tearDown(self):
        """
        Callback will be called after run application
        """
        pass

    def addOptions(self, parser):
        """
        Add options to command line interface

        :type parser: optparse.OptionParser
        """
        pass

    @property
    def name(self):
        """
        Application name
        """
        return self.__name

    @property
    def options(self):
        """
        Command line options
        """
        if self.__options is None:
            raise RuntimeError(
                'Options not found. Working outside initialize callback.',
            )

        return self.__options

    @options.setter
    def options(self, options):
        assert isinstance(options, self.config_class),\
            'options is not instance of "{}"'.format(self.config_class.__name__)

        if self.is_master_app:
            for sub_app in self.__sub_apps:
                sub_app.options = options

        self.__options = options

    @property
    def context(self):
        """
        Application context instance

        :rtype: noseapp.app.context.AppContext
        """
        return self.__context

    @property
    def plugins(self):
        """
        Plugins list
        """
        return self.__context.plugins

    @property
    def is_sub_app(self):
        """
        Flag to definition: that's sub application?
        """
        return self.__is_sub_app

    @property
    def is_master_app(self):
        """
        Flag to definition: that's master application?
        """
        return not self.__is_sub_app

    @property
    def suites(self):
        """
        Suites list
        """
        return self.__context.suites

    @property
    def sub_apps(self):
        """
        Sub application list
        """
        return self.__sub_apps

    @property
    def status(self):
        """
        Status of application
        """
        return 'sub' if self.__is_sub_app else 'master'

    def add_before(self, func):
        """
        Add callback to setup
        """
        self.__context.add_setup(func)
        return func

    def add_after(self, func):
        """
        Add callback to teardown
        """
        self.__context.add_teardown(func)
        return func

    @staticmethod
    def shared_extension(name=None, cls=None, args=None, kwargs=None):
        """
        Shared extension to Suites and TestCases.
        Use require param on noseapp.suite.Suite class for connect.

        :param name: extension name
        :type name: str

        :param cls: extension class
        :param args, kwargs: class init arguments

        :type args: tuple
        :type kwargs: dict
        """
        if cls is None:
            raise ValueError('cls param is required')

        if name is None and not hasattr(cls, 'name'):
            raise AttributeError(
                '"{}" does not have "name" attribute'.format(cls.__name__),
            )
        elif name is None:
            name = cls.name

        logger.debug('Shared extension "%s"', name)

        return extensions.set(name, cls, to_transport=True, args=args, kwargs=kwargs)

    @staticmethod
    def shared_data(name, data):
        """
        Shared extension to TestCase classes.
        Data will be copied during installation.

        :param name: this is property name of case class
        :type name: str

        :param data: any object
        """
        logger.debug('Shared data "%s"', name)

        return extensions.set(name, data, to_transport=False)

    def register_suite(self, suite, add_prefix=True):
        """
        Add suite in application

        :type suite: noseapp.suite.base.Suite
        """
        logger.debug('Register suite "%s"', suite.name)

        suite.mount_to_app(self)

    def register_suites(self, suites):
        """
        App suite list in application

        :type suites: iter
        """
        for suite in suites:
            self.register_suite(suite)

    def load_suites(self, path, recursive=True, merge_suites=False):
        """
        Auto load suites. Path can be package or simple dir.

        :param path: dir path
        :type path: str

        :param recursive: recursive load from path or load from simple dir
        :type recursive: bool

        :param merge_suites: merge suites from sub application
         to self context. For master application only.
        :type merge_suites: bool
        """
        if path not in sys.path:
            sys.path.append(path)

        if recursive:
            suites = loader.load_suites_from_path(path)
        else:
            suites = loader.load_suites_from_dir(path)

        self.register_suites(suites)

        if merge_suites:
            merge_context(self, merge_suites=True)

    def run(self):
        """
        To perform run suites
        """
        logger.debug('Run application %s', repr(self))

        if self.__is_sub_app:
            raise RuntimeError(
                'Application {} can not be run. Application is sub application.'.format(self),
            )

        @contextmanager
        def teardown_app():
            try:
                yield
            finally:
                app_callback(self, 'tearDownApp')

        with teardown_app():
            success = self.__test_program.run()

        return success

    def __repr__(self):
        return '<NoseApp({}): {}>'.format(
            self.status, self.__name,
        )



# -*- coding: utf-8 -*-

"""
Auto load suites for register in noseapp.base.NoseApp
"""

import os
import logging
from importlib import import_module


logger = logging.getLogger(__name__)


TEST_NAME_PREFIX = 'test'
DEFAULT_TEST_NAME = 'runTest'


class LoaderError(BaseException):
    pass


def check_path_is_exist(path):
    """
    Check exist path
    """
    if not os.path.exists(path):
        raise LoaderError('Dir "{}" is not exist'.format(path))


def is_package(path):
    """
    Maybe is path python package?
    """
    return os.path.isfile(
        os.path.join(path, '__init__.py'),
    )


def load_suites_from_dir(path, import_base=None):
    """
    Load suites from dir

    :type path: str
    :param import_base: base import path
    :type import_base: str
    """
    from noseapp.suite.base import Suite

    logger.debug('Try to load suites from dir: "%s"', path)

    suites = []

    if import_base and not is_package(path):
        return suites

    py_files = filter(
        lambda f: f.endswith('.py') and not f.startswith('_'),
        os.listdir(path),
    )
    modules = (m.replace('.py', '') for m in py_files)

    for module_name in modules:
        if import_base:
            module_name = '{}.{}'.format(import_base, module_name)

        logger.debug('Import python module: "%s"', module_name)

        module = import_module(module_name)

        module_suites = (
            getattr(module, atr)
            for atr in dir(module)
            if isinstance(
                getattr(module, atr, None), Suite,
            )
        )

        suites.extend(module_suites)

    return suites


def load_suites_from_path(path, import_base=None):
    """
    Recursive load suites from path

    :param path: path to dir
    :type path: str
    :param import_base: base import path
    :type import_base: str
    """
    logger.debug('Load suites from path: "%s"', path)

    check_path_is_exist(path)

    suites = []
    copy_import_base = import_base

    suites.extend(
        load_suites_from_dir(path, import_base=import_base),
    )

    for root, dirs, files in os.walk(path):
        for d in dirs:
            dir_abs_path = os.path.join(root, d)

            if import_base is None:
                _import_base = d
            else:
                _import_base = '{}.{}'.format(import_base, d)

            suites.extend(
                load_suites_from_path(
                    dir_abs_path,
                    import_base=_import_base,
                ),
            )

            import_base = copy_import_base

    return suites


def load_tests_from_test_case(
        test_case_class,
        method_name=None,
        test_name_prefix=TEST_NAME_PREFIX,
        default_test_name=DEFAULT_TEST_NAME):
    """
    Create instances of test_case_class by test names
    """
    logger.debug('Load test from test case: "%s"', test_case_class.__name__)

    cls_dir = dir(test_case_class)

    if method_name:
        if method_name not in cls_dir:
            raise LoaderError(
                'Method "{}" of "{}" class is not found'.format(
                    method_name, test_case_class.__name__,
                ),
            )
        return map(test_case_class, [method_name])

    def is_test_name(name):
        return name.startswith(test_name_prefix) \
            or \
            name == default_test_name

    return map(test_case_class, filter(is_test_name, cls_dir))


def load_suite_by_name(name, suites):
    """
    Find suite in list by name

    :param name: suite name
    :param suites: suites list
    """
    logger.debug('Load suite "%s" from list', name)

    for suite in suites:
        if suite.name == name:
            return suite
    else:
        raise LoaderError(
            'Suite "{}" is not found'.format(name),
        )


def load_case_from_suite(class_name, suite):
    """
    Find test case class by class name in suite or suite mediator

    :param class_name: class name
    :param suite: suite or suite mediator instance
    """
    logger.debug('Load test case "%s" from suite "%s"', class_name, suite.name)

    for case in suite.test_cases:
        if case.__name__ == class_name:
            return case
    else:
        raise LoaderError(
            'Test case "{}" is not found'.format(class_name),
        )



# -*- coding: utf-8 -*-


class SuiteContext(object):

    def __init__(self, require):
        self.__require = require

        self.__setup = []
        self.__teardown = []

        self.__pre_run_handlers = []
        self.__post_run_handlers = []

        self.__test_cases = []
        self.__extensions = {}

    @property
    def pre_run_handlers(self):
        return self.__pre_run_handlers

    @property
    def post_run_handlers(self):
        return self.__post_run_handlers

    @property
    def require(self):
        return self.__require

    @property
    def test_cases(self):
        return self.__test_cases

    @property
    def extensions(self):
        return self.__extensions

    def add_test_case(self, case):
        self.__test_cases.append(case)

    def add_extension(self, name, ext):
        self.__extensions[name] = ext

    def add_pre_run_handler(self, func):
        self.__pre_run_handlers.append(func)

    def add_post_run_handler(self, func):
        self.__post_run_handlers.append(func)

    def add_setup(self, func):
        self.__setup.append(func)

    def add_teardown(self, func):
        self.__teardown.append(func)

    def setup(self):
        """
        Callback for nose context
        """
        for callback in self.__setup:
            callback()

    def teardown(self):
        """
        Callback for nose context
        """
        for callback in self.__teardown:
            callback()



# -*- coding: utf-8 -*-

import time
from contextlib import contextmanager

from nose.core import TextTestRunner as _TextTestRunner
from nose.result import TextTestResult as _TextTestResult


class TextTestResult(_TextTestResult):
    pass


@contextmanager
def setup_teardown(suites):
    suites.setUp()
    try:
        yield
    finally:
        suites.tearDown()


class RunPerformer(object):

    def __init__(self, runner):
        self.runner = runner

    def __call__(self, suites, result):
        """
        :type suites: noseapp.core.suite.base.BaseSuite
        :type result: noseapp.core.runner.base.TextTestResult
        """
        with setup_teardown(suites):
            for suite in suites:
                suite(result)


class BaseTestRunner(_TextTestRunner):

    result_class = TextTestResult
    performer_class = RunPerformer

    def _makeResult(self):
        return self.result_class(
            self.stream,
            self.descriptions,
            self.verbosity,
            self.config,
        )

    def _makePerformer(self):
        return self.performer_class(self)

    def run(self, suites):
        wrapper = self.config.plugins.prepareTest(suites)
        if wrapper is not None:
            suites = wrapper

        wrapped = self.config.plugins.setOutputStream(self.stream)
        if wrapped is not None:
            self.stream = wrapped

        result = self._makeResult()
        performer = self._makePerformer()

        start = time.time()
        with setup_teardown(suites):
            performer(suites, result)
        stop = time.time()

        result.printErrors()
        result.printSummary(start, stop)

        self.config.plugins.finalize(result)

        return result


__all__ = (
    RunPerformer,
    TextTestResult,
    BaseTestRunner,
)



# -*- coding: utf-8 -*-

from unittest import TestCase


class SelfTest(TestCase):
    """
    Test of full cycle:
        * Create application
        * Call to callbacks
        * Config data
        * Tests to run
        * Shared extensions
        * Shared data

    Test given guarantee to application is stable working,
    but that's not means for test program api
    """

    def setUp(self):
        from testapp.app import create_app

        self.app = create_app(exit=False)
        self.assertTrue(self.app.run())

    def check_config(self):
        self.assertEqual(self.app.config.DEFAULT, 1)
        self.assertEqual(self.app.config.CONSTANT, 2)

    def check_callbacks(self):
        self.assertTrue(self.app.self_test.add_options_called)

        self.assertTrue(self.app.self_test.setup_app_called)
        self.assertTrue(self.app.self_test.teardown_app_called)

        self.assertTrue(self.app.self_test.setup_app_context_called)
        self.assertTrue(self.app.self_test.teardown_app_context_called)

        self.assertTrue(self.app.self_test.setup_suite_context_called)
        self.assertTrue(self.app.self_test.teardown_suite_context_called)

        self.assertTrue(self.app.self_test.setup_case_called)
        self.assertTrue(self.app.self_test.teardown_case_called)

        self.assertTrue(self.app.self_test.setup_case_class_called)
        self.assertTrue(self.app.self_test.teardown_case_class_called)

        self.assertTrue(self.app.self_test.suite_before_called)
        self.assertTrue(self.app.self_test.suite_after_called)
        self.assertTrue(self.app.self_test.pre_run_case_called)
        self.assertTrue(self.app.self_test.post_run_case_called)

    def check_was_worked(self):
        self.assertGreater(len(self.app.self_test.worked_cases), 0)

    def runTest(self):
        self.check_config()
        self.check_callbacks()
        self.check_was_worked()



# -*- coding: utf8 -*-

from __future__ import absolute_import

from multiprocessing import cpu_count
from multiprocessing.pool import ThreadPool

from noseapp.core.suite.base import BaseSuite
from noseapp.core.suite.base import SuitePerformer


class ThreadSuitePerformer(SuitePerformer):
    """
    Run tests with multiprocessing.pool.ThreadPool
    """

    def __call__(self, pool=None):
        self_pool = not bool(pool)
        size = self.suite.config.options.async_tests

        if size < 0:
            size = cpu_count() / 2

        pool = pool or ThreadPool(int(round(size)) or 2)

        for test in self.suite.tests:
            if self.result.shouldStop:
                break

            if isinstance(test, BaseSuite):
                test.run(self.result, pool=pool)
                continue

            pool.apply_async(self.run_one_test, args=(test, ))

        if self_pool:
            pool.close()
            pool.join()



# -*- coding: utf-8 -*-

from unittest import TestCase as BaseTestCase


def make_test_case_class_from_function(
        func,
        base_class,
        doc=None,
        simple=False,
        class_name=None,
        class_name_creator=None):
    """
    It's just create test case class from function.
    Function will be bound method (runTest) of new class.

    :param func: test function. for example: my_test = lambda case: 'just do it something'
    :param base_class: base class for making class
    :param simple: if true that not inject case instance
    :param class_name: name of new class. default function name
    :param class_name_creator: callable object for creating name of function object
    """
    if callable(class_name_creator):
        class_name = class_name_creator(func)

    cls = type(
        class_name or func.__name__,
        (base_class, ),
        {
            '__doc__': doc or func.__doc__,
        },
    )

    if simple:
        cls.runTest = lambda s: func()
    else:
        cls.runTest = func

    return cls


def case_is_mount(case):
    """
    If case is mounting to suite then True else False

    :param case: test case class or instance

    :rtype: bool
    """
    return hasattr(case, '_of_suite')


def get_case_master_id(case):
    """
    Get id of master process

    :type case: ToNoseAppTestCase
    """
    if hasattr(case, 'test'):
        case = case.test

    return case._ToNoseAppTestCase__master_id


class ExtensionStorage(object):
    """
    Storage for extensions of test case
    """

    def __init__(self, suite):
        """
        :type suite: noseapp.suite.base.Suite
        """
        self.__require = []
        self.__require.extend(suite.require)

        self.__extensions = dict(
            (k, v) for k, v in suite.context.extensions.items()
        )

    def __call__(self, name):
        if name in self.__require:
            return self.__extensions[name]

        raise LookupError(
            'Extension "{}" is not required'.format(name),
        )


class ToNoseAppTestCase(object):
    """
    This is mixin for NoseApp supporting
    """

    def __init__(self, *args, **kwargs):
        super(ToNoseAppTestCase, self).__init__(*args, **kwargs)

        self.__master_id = id(self)

    @classmethod
    def mount_to_suite(cls, suite):
        """
        Create class of suite

        :type suite: noseapp.suite.base.Suite
        """
        if case_is_mount(cls):
            raise RuntimeError(
                '"{}" already mount to {}'.format(
                    cls.__name__, cls._of_suite,
                ),
            )

        setattr(cls, '_of_suite', suite.name)
        setattr(cls, '_ext_storage', ExtensionStorage(suite))

        return cls

    @property
    def of_suite(self):
        if case_is_mount(self):
            return self._of_suite

        return None

    def ext(self, name):
        if case_is_mount(self):
            return self._ext_storage(name)

        return None

    def __str__(self):
        if case_is_mount(self):
            return '{} ({}:{})'.format(
                self._testMethodName,
                self._of_suite,
                self.__class__.__name__,
            )

        return super(ToNoseAppTestCase, self).__str__()


class TestCase(ToNoseAppTestCase, BaseTestCase):
    """
    Base case class
    """
    pass



# -*- coding: utf8 -*-

from __future__ import absolute_import

from multiprocessing import cpu_count
from multiprocessing.pool import ThreadPool

from noseapp.core.runner.base import RunPerformer


class ThreadingRunPerformer(RunPerformer):
    """
    Run tests with multiprocessing.pool.ThreadPool
    """

    def __call__(self, suites, result):
        size = self.runner.config.options.async_suites

        if size < 0:
            size = cpu_count()

        pool = ThreadPool(size)

        for suite in suites:
            pool.apply_async(suite, args=(result,))

        pool.close()
        pool.join()



# -*- coding: utf-8 -*-

from nose.plugins import Plugin as BasePlugin


class AppPlugin(BasePlugin):
    """
    Base application plugin
    """

    def __init__(self):
        super(AppPlugin, self).__init__()

        self.app = None

    def init_app(self, app):
        """
        Push app instance

        :type app: noseapp.NoseApp
        """
        self.app = app



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from multiprocessing import cpu_count

from gevent.pool import Pool

from noseapp.core.runner.base import RunPerformer


class GeventRunPerformer(RunPerformer):
    """
    Run suites with gevent
    """

    def __call__(self, suites, result):
        size = self.runner.config.options.async_suites

        if size < 0:
            size = cpu_count()

        pool = Pool(size)

        for suite in suites:
            pool.spawn(suite, result)

        pool.join()



# -*- coding: utf-8 -*-

from unittest import TestCase


class TestDefaultClasses(TestCase):
    """
    Test default classes in NoseApp class property
    """

    def runTest(self):
        from noseapp import NoseApp
        from noseapp.core import TestProgram
        from noseapp.app.config import Config
        from noseapp.core.factory import ClassFactory
        from noseapp.core.collector import CollectSuite

        self.assertEqual(NoseApp.config_class, Config)
        self.assertEqual(NoseApp.program_class, TestProgram)
        self.assertEqual(NoseApp.class_factory, ClassFactory)
        self.assertEqual(NoseApp.collector_class, CollectSuite)


class TestAppName(TestCase):

    def runTest(self):
        from noseapp import NoseApp

        app = NoseApp('hello')
        self.assertEqual(app.name, 'hello')
